local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110228170935') == 'number' and '20110228170935' or date('%Y%m%d%H%M%S')

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:NewAddon(MAJOR_VERSION, 'AceConsole-3.0' --[[,'AceEvent-3.0']] ,'AceHook-3.0')
--local L = LibStub('AceLocale-3.0'):GetLocale(MAJOR_VERSION)

local UnitXP = _G.UnitXP
local UnitXPMax = _G.UnitXPMax
local UnitLevel = _G.UnitLevel
local GetMaxPlayerLevel = _G.GetMaxPlayerLevel
local GetXPExhaustion = _G.GetXPExhaustion

--- Setup debug stuff.
do
    local libDebugLog = LibStub:GetLibrary('LibDebugLog-1.0',true)
    if libDebugLog then
        libDebugLog:Embed(addon)
    else
        function addon:Debug (...)
        end
    end
end

local chatShort = "glb"
local chatLong = MAJOR_VERSION
local addonDbName = MAJOR_VERSION .. 'DB'

addon.optionsTable = {
    name = MAJOR_VERSION,
    handler = addon,
    type = 'group',
    args = {
    },
}

addon.defaults = {
    profile = {
    },
}

--- Called for chat commands.
-- @param input the chat command
function addon:ChatCommand(input)
    if not input or input:trim() == '' then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub('AceConfigCmd-3.0').HandleCommand(MAJOR_VERSION, chatShort, MAJOR_VERSION, input)
    end
end

--- Called when the addon is loaded
-- (Ace3 does this)
function addon:OnInitialize()
    self.db = LibStub('AceDB-3.0'):New(addonDbName, addon.defaults)
    LibStub('AceConfig-3.0'):RegisterOptionsTable(MAJOR_VERSION, self.optionsTable)
    self.optionsFrame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(MAJOR_VERSION, MAJOR_VERSION)
    self:RegisterChatCommand(chatShort, 'ChatCommand')
    self:RegisterChatCommand(chatLong, 'ChatCommand')

    self:sanityCheck()
end

function addon:sanityCheck()
    -- sanitiy check
    unitXPMax = UnitXPMax('player')
    if UnitXPMax('player') ~= 0 and unitXPMax ~= XP_PER_LEVEL[UnitLevel('player')] then
        error("The XP database is incorrect, please report this. Expected XP for level "
           .. UnitLevel('player') .. " is " .. XP_PER_LEVEL[UnitLevel('player')] .. " but UnitXPMax returned "
           .. unitXPMax)
    end
end


--- Called when the addon is enabled.
function addon:OnEnable()
end

--- Called when the addon is disabled
function addon:OnDisable()
end

function addon:merge(t1, t2)
    for k,v in pairs(t2) do
        if t1[k] and type(t1[k]) == 'table' and type(v) == 'table' then
            t1[k] = self:merge(t1[k],v)
        else
            t1[k] = v
        end
    end
    return t1
end
